<?php
/*
 * @copyright 2006 M3 Media Services
 * @version $Id:Field.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 * @since 0.3.19 - Feb 10, 2006
 *
 */

require_once MAX_PATH . '/lib/max/Dal/Reporting.php';
require_once MAX_PATH . '/lib/max/DateRange.php';
require_once MAX_PATH . '/lib/max/Admin/Reporting/ZoneScope.php';

/**
 * Abstract data field object, used to supply reports with parameters.
 *
 * Always use the factory method to instantiate fields -- it will create
 * the right subclass for you.
 *
 * @abstract
 */
class Field
{
    var $tabindex;
    var $tabkeeper;

    /**
     * Creates a new Field object of the appropriate subclass.
     *
     * @static
     */
    /* static */ function factory($field_type)
    {
        // Text field
        if ($field_type == 'edit')
        {
            $product = new TextField();
        }

        // CampaignID-dropdown
        elseif ($field_type == 'campaignid-dropdown')
        {
            $product = new CampaignIdField();
        }

        // ClientID-dropdown
        elseif ($field_type == 'clientid-dropdown' || $field_type == 'advertiser')
        {
            $product = new AdvertiserIdField();
        }

        // AffiliateID-dropdown
        elseif ($field_type == 'affiliateid-dropdown' || $field_type == 'publisher')
        {
            $product = new PublisherIdField();
        }

        // Tracker-dropdown
        elseif ($field_type == 'trackerid-dropdown') {
            $product = new TrackerField();
        }

        // ZoneID-dropdown
        elseif ($field_type == 'zoneid-dropdown')
        {
            $product = new ZoneIdField();
        }

        elseif ($field_type == 'zone-scope')
        {
            $product = new ZoneScopeField();
        }

        elseif ($field_type == 'date-range' || $field_type == 'date-month')
        {
            $product = new DateRangeField(new Date());
        }

        elseif ($field_type == 'scope')
        {
            $product = new ScopeSelectionField();
        }

        else
        {
            MAX::raiseError("The report module discovered a field type that it didn't know how to handle.", MAX_ERROR_INVALIDARGS);
        }

        $product->tabindex = 0;
        $product->tabkeeper =& $product;

        return $product;
    }

    /**
     * Return a single value based on a $_GET-style array of values
     *
     */
    function getPassableValueFromQueryArray($values, $base_key)
    {
        return $values[$base_key];
    }

    /**
     * Echo HTML representing the appropriate field type
     *
     * @abstract
     */
    function displayField($fields, $key)
    {
        MAX::raiseError('Programmer failure: "displayField" is an abstract method, and must be over-riden.', MAX_ERROR_NOMETHOD);
    }

    function displayExtrasAfterForm($fields, $key)
    {
    }

    function _displayFieldHeader($title)
    {
        echo "<tr><td width='30'>&nbsp;</td>";
        echo "<td width='200'>".$title."</td>";
        echo "<td width='370'>";
    }

    function _displayFieldFooter()
    {
        echo "</td>";
        echo "</tr>";
        echo "<tr><td width='30'><img src='images/spacer.gif' height='1' width='100%'></td>";
        echo "<td colspan='2'><img src='images/break-l.gif' height='1' width='200' vspace='6'></td></tr>";
    }
}

/**
 * A plain, single-line text-entry field used as a parameter for reports.
 */
class TextField extends Field
{
    function displayField($fields, $key)
    {
        $this->_displayFieldHeader($fields[$key]['title']);
        echo "<input type='text' name='".$key."' size='";
        echo isset($fields[$key]['size']) ? $fields[$key]['size'] : "";
        echo "' value='";
        echo isset($fields[$key]['default']) ? $fields[$key]['default'] : "";
        echo "' tabindex='".($this->tabkeeper->tabindex++)."'>";
        $this->_displayFieldFooter();
    }
}

class CampaignIdField extends Field
{
    function displayField($fields, $key)
    {
        $dal = new MAX_Dal_Reporting();

        $this->_displayFieldHeader($fields[$key]['title']);
        echo "<select name='".$key."' tabindex='".($this->tabkeeper->tabindex++)."'>";

        $campaignArray = $dal->phpAds_getCampaignArray();
        foreach (array_keys($campaignArray) as $ckey)
            echo "<option value='".$ckey."'>".$campaignArray[$ckey]."</option>";
        echo "</select>";
        $this->_displayFieldFooter();
    }
}

class AdvertiserIdField extends Field
{
    function displayField($fields, $key)
    {
        $dal = new MAX_Dal_Reporting();

        if (phpAds_isUser(phpAds_Advertiser))
        {
            echo "<input type='hidden' name='".$key."' value='".phpAds_getUserID()."'>";
        }
        else
        {
            $this->_displayFieldHeader($fields[$key]['title']);
            echo "<select name='".$key."' tabindex='".($this->tabkeeper->tabindex++)."'>";

            $clientArray = $dal->phpAds_getAdvertiserArray();
            foreach (array_keys($clientArray) as $ckey)
                echo "<option value='".$ckey."'>".$clientArray[$ckey]."</option>";
            echo "</select>";
            $this->_displayFieldFooter();
        }

    }
}

class PublisherIdField extends Field
{
    function displayField($fields, $key)
    {
        $dal = new MAX_Dal_Reporting();

        if (phpAds_isUser(phpAds_Publisher))
        {
            echo "<input type='hidden' name='".$key."' value='".phpAds_getUserID()."'>";
        }
        else
        {
            $this->_displayFieldHeader($fields[$key]['title']);
            echo "<select name='".$key."' tabindex='".($this->tabkeeper->tabindex++)."'>";

            $affiliateArray = $dal->phpAds_getPublisherArray();
            foreach (array_keys($affiliateArray) as $ckey)
                echo "<option value='".$ckey."'>".$affiliateArray[$ckey]."</option>";
            echo "</select>";
            $this->_displayFieldFooter();
        }
    }
}

class TrackerField extends Field
{
    function displayField($fields, $key)
    {
        $dal = new MAX_Dal_Reporting();

        $this->_displayFieldHeader($fields[$key]['title']);

        echo "<select name='".$key."' tabindex='".($this->tabkeeper->tabindex++)."'>";

        $trackerArray = $dal->phpAds_getTrackerArray();
        for (reset($trackerArray);$ckey=key($trackerArray);next($trackerArray))
            echo "<option value='".$ckey."'>".$trackerArray[$ckey]."</option>";
        echo "</select>";
        $this->_displayFieldFooter();
    }
}

class ZoneIdField extends Field
{
    function displayField($fields, $key)
    {
        $this->_displayFieldHeader($fields[$key]['title']);

        echo "<select name='".$key."' tabindex='".($this->tabkeeper->tabindex++)."'>";
        $this->displayZonesAsOptionList();
        echo "</select>";

        $this->_displayFieldFooter();
    }

    function displayZonesAsOptionList()
    {
        $dal = new MAX_Dal_Reporting();

        $zoneArray = $dal->phpAds_getZoneArray();
        foreach (array_keys($zoneArray) as $ckey)
            echo "<option value='".$ckey."'>".$zoneArray[$ckey]."</option>";
    }
}

class ZoneScopeField extends ZoneIdField
{
    function displayZonesAsOptionList()
    {
        echo "<option value='all'>(All available zones)</option>";
        parent::displayZonesAsOptionList();
    }

    function getPassableValueFromQueryArray($values, $base_key)
    {
        $scope = new ZoneScope();
        $zone_id = $values[$base_key];
        if ($zone_id == 'all') {
            $scope->useAllZones();
        } else {
            $scope->useZoneId($zone_id);
        }
        return $scope;
    }
}

class DateRangeField extends Field
{
    /* @var bool */
    var $_week;
    /* @var Date */
    var $_now;
    /* @var DateRange */
    var $_range;

    function __construct($now)
    {
        $this->_week = false;
        $this->_now = $now;
        $this->_range = new DateRange($now);
    }

    function DateRangeField($now)
    {
        $this->__construct($now);
    }

    function isLastMonth()
    {
        if ($this->_week)
        {
            return false;
        }
        return true;
    }

    function isLastWeek()
    {
        return $this->_week;
    }

    function useLastWeek()
    {
        $this->_range->useLastWeek();
        $this->_week = true;
    }

    function getStartDate()
    {
        return $this->_range->getStartDate();
    }

    function getEndDate()
    {
        return $this->_range->getEndDate();
    }

    function displayField($fields, $key)
    {
        $this->_displayFieldHeader($fields[$key]['title']);
        $overall_name = $key;
        $preset_id = $overall_name . '_preset';
        $start_id = $overall_name . '_start';
        $end_id = $overall_name . '_end';
        $start_button_id = $start_id . '_button';
        $end_button_id = $end_id . '_button';
?>
    <fieldset class='date-range'>
        <legend>Date range</legend>

        <select name='<?= $preset_id?>' id='<?= $preset_id?>'>
            <option value='lastmonth' selected='selected'>Last month</option>
            <option value='lastweek'>Last week</option>
            <option value='last7days'>Last 7 days</option>
            <option value='thismonth'>This month</option>
            <option value='thisweek'>This week</option>
            <option value='specific'>Specific dates</option>
        </select>

        <fieldset class='actual-dates'>
        <legend>Actual dates</legend>
            <label for='<?= $start_id ?>'>From</label>
            <input class='date' name='<?= $start_id ?>' id='<?= $start_id ?>' type='text' value=''/>
            <button id='<?= $start_button_id ?>'>...</button>

            <label for='<?= $end_id ?>'>To</label>
            <input class='date' name='<?= $end_id ?>' id='<?= $end_id ?>' type='text' value='' />
            <button id='<?= $end_button_id ?>'>...</button>
        </fieldset>
    </fieldset>

<?
        $this->_displayFieldFooter();
    }

    function displayExtrasAfterForm($fields, $key)
    {
        $overall_name = $key;
        $preset_id = $overall_name . '_preset';
        $start_id = $overall_name . '_start';
        $end_id = $overall_name . '_end';
        $start_button_id = $start_id . '_button';
        $end_button_id = $end_id . '_button';
        ?>
    <script type='text/javascript'>
        Calendar.setup({
            inputField : '<?= $start_id ?>',
            ifFormat   : '%Y-%m-%d',
            button     : '<?= $start_button_id ?>'
        })
        Calendar.setup({
            inputField : '<?= $end_id ?>',
            ifFormat   : '%Y-%m-%d',
            button     : '<?= $end_button_id ?>'
        })
    </script>

        <?
    }

    function getPassableValueFromQueryArray($values, $base_key)
    {
        $range = new DateRange($this->_now);
        $range->useValuesFromQueryArray($values, $base_key);
        return $range;
    }

    function getRepresentativeValue()
    {
        return $this->_range;
    }
}

class ScopeSelectionField extends Field
{
    function displayField($fields, $key)
    {
        $this->_displayFieldHeader($fields[$key]['title']);

        $overall_name = $key;
        $entity_name = $overall_name . '_entity';
        $publisher_name = $overall_name . '_publisher';

        ?>

  <input type="radio" name="<?= $entity_name ?>" id="<?= $entity_name ?>" value="advertiser">A specific advertiser</input>
  <select name="advertiser">
    <option value="1">Advertiser 1</option>
    <option value="2">Advertiser 2</option>
    <option value="3">Advertiser 3</option>
  </select>

  <br />
  <input type="radio"name="<?= $entity_name ?>" id="<?= $entity_name ?>" value="publisher">A specific publisher</input>

  <select name="<?= $publisher_name ?>" id="<?= $publisher_name ?>" ">
    <option value="1">Publisher 1</option>
    <option value="2">Publisher 2</option>
    <option value="3">Publisher 3</option>
  </select>
  <br />

  <input type="radio" name="<?= $entity_name ?>" id="<?= $entity_name ?>" value="all" checked="checked">All available data</input>

        <?php
        $this->_displayFieldFooter();
    }

    function getPassableValueFromQueryArray($values, $base_key)
    {
        $scope = new ReportScope();
        $scope->useValuesFromQueryArray($values, $base_key);
        return $scope;
    }
}
?>