<?php
/**
 * Zone selection scope for Max Media Manager
 *
 * A ZoneScope object represents either all available zones or
 * one specific zone.
 *
 * This is useful for reports, where a user may want to
 * report on everything or narrow it down to one zone.
 *
 * @todo Consider support for multiple zones.
 * @since 0.3.19 - Feb 24, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id: ZoneScope.php 4307 2006-02-24 16:45:08Z roh@m3.net $
 */

class ZoneScope
{
    /* @var bool */
    var $_is_zone_specified;

    /* @var int */
    var $_zone_id;

    /**
     * Does this scope refer to just one zone?
     *
     * @return bool False if this scope refers to all zones
     */
    function isSpecificZone()
    {
        return $this->_is_zone_specified;
    }

    /**
     * The numeric zone identifier that this scope refers to.
     *
     * @return int The specific zone ID.
     */
    function getZoneId()
    {
        if ($this->_is_zone_specified) {
            return $this->_zone_id;
        }
        trigger_error('Programmer failure: Max was asked to provide a single zone ID to represent all zones.');
    }

    function useZoneId($zone_id)
    {
        $this->_is_zone_specified = true;
        $this->_zone_id = $zone_id;
    }

    function useAllZones()
    {
        $this->_is_zone_specified = false;
    }
}
?>
