<?php
/**
 * Test report generation for Max Media Manager
 *
 * @since 0.3.19 - Feb 14, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id: ReportExecuteModule.admin.test.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 */

require_once(MAX_PATH . '/lib/max/Admin/Reporting/ExecuteModule.php');
require_once MAX_PATH . '/plugins/reports/Reports.php';

class ReportExecuteTest extends UnitTestCase
{
    function testSimpleInfo()
    {
        $import = array(
                'example' => array(
                    'title' => 'Simple text',
                    'type' => 'edit',
                ),
            );
        $info = array('plugin-import' => $import);
        $passed_in = array('example' => 'simple_value');
        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $this->assertEqual($variables[0], "simple_value");
    }

    function testSimpleInfo_Missing()
    {
        $import = array(
                'example' => array(
                    'title' => 'Empty text',
                    'type' => 'edit',
                ),
            );
        $info = array('plugin-import' => $import);
        $passed_in = array(); //no values
        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $this->assertEqual($variables[0], '');
    }

    function testDateRange_MissingPreset()
    {
        $import = array(
                'example' => array(
                    'title' => 'Date range',
                    'type' => 'date-range',
                ),
            );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_start' => '1998-07-06',
            'example_end' => '1998-08-08'
        );
        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('1998-07-06'));
        $this->assertEqual($range->getEndDate(), new Date('1998-08-08'));
    }

    function testDateRange_Specific()
    {
        $import = array(
                'example' => array(
                    'title' => 'Date range',
                    'type' => 'date-range',
                ),
            );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'specific',
            'example_start' => '1998-07-06',
            'example_end' => '1998-08-08'
        );
        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('1998-07-06'));
        $this->assertEqual($range->getEndDate(), new Date('1998-08-08'));
    }

    function testDateRange_ThisWeek()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'thisweek',
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('2000-06-11 00:00:00'));
        $this->assertEqual($range->getEndDate(), new Date('2000-06-18 00:00:00'));
    }

    function testDateRange_LastWeek()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'lastweek',
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('2000-06-04 00:00:00'));
        $this->assertEqual($range->getEndDate(), new Date('2000-06-11 00:00:00'));
    }

    function testDateRange_Last7Days()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'last7days',
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('2000-06-09 00:00:00'));
        $this->assertEqual($range->getEndDate(), new Date('2000-06-16 00:00:00'));
    }

    function testDateRange_LastMonth()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'lastmonth',
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('2000-05-01 00:00:00'));
        $this->assertEqual($range->getEndDate(), new Date('2000-06-01 00:00:00'));
    }

    function testDateRange_ThisMonth()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'thismonth',
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertEqual(count($variables), 1);
        $range = $variables[0];
        $this->assertIsA($range, 'DateRange');
        $this->assertEqual($range->getStartDate(), new Date('2000-06-01 00:00:00'));
        $this->assertEqual($range->getEndDate(), new Date('2000-07-01 00:00:00'));
    }

    function testDateRange_MissingEverything()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            // nothing passed in
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertError();
    }

    function testDateRange_FunnyPreset()
    {
        $import = array(
            'example' => array(
                'title' => 'Date range',
                'type' => 'date-range',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_preset' => 'This_Is_Not_A_Real_Specifier'
        );

        $module = new ReportExecuteModule();
        $module->now = new Date('2000-06-16 16:16:16');
        $variables = $module->_getVariablesForReport($info, $passed_in);
        $this->assertError();
    }

    function testEntity_All()
    {
        $import = array(
            'example' => array(
                'title' => 'Report on',
                'type' => 'scope',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_entity' => 'all'
        );

        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);

        $this->assertEqual(count($variables), 1);
        $scope = $variables[0];
        $this->assertIsA($scope, 'ReportScope');
        $this->assertEqual($scope->description, 'all available advertisers and publishers');
    }

    function testEntity_Publisher()
    {
        $import = array(
            'example' => array(
                'title' => 'Report on',
                'type' => 'scope',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_entity' => 'publisher',
            'example_publisher' => '316',
        );

        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);

        $this->assertEqual(count($variables), 1);
        $scope = $variables[0];
        $this->assertIsA($scope, 'ReportScope');
        $this->assertEqual($scope->description, 'publisher 316');
        $publisher_id = $scope->getPublisherId();
        $this->assertEqual($publisher_id, 316);
    }

    function testScope_Advertiser()
    {
        $import = array(
            'example' => array(
                'title' => 'Report on',
                'type' => 'scope',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_entity' => 'advertiser',
            'example_advertiser' => '420',
        );

        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);

        $this->assertEqual(count($variables), 1);
        $scope = $variables[0];
        $this->assertIsA($scope, 'ReportScope');
        $this->assertEqual($scope->description, 'advertiser 420');
        $publisher_id = $scope->getAdvertiserId();
        $this->assertEqual($publisher_id, 420);
    }

    function testScope_FunnyEntity()
    {
        $import = array(
            'example' => array(
                'title' => 'Report on',
                'type' => 'scope',
            ),
        );
        $info = array('plugin-import' => $import);
        $passed_in = array(
            'example_entity' => 'This_Is_Not_A_Valid_Choice'
        );

        $module = new ReportExecuteModule();
        $variables = $module->_getVariablesForReport($info, $passed_in);

        $this->assertError();
    }

    function testExecuteLegacy_Numbers()
    {
        Mock::generatePartial('Plugins_Reports', 'MockReportPlugin_ForExecuteModule_NumberTest', array('execute'));

        $module = new ReportExecuteModule();
        $report = new MockReportPlugin_ForExecuteModule_NumberTest($this);
        $report->expectOnce('execute', array(1, 2, 3));
        $passed_in = array(1, 2, 3);

        $module->_executeReportWithVariables($report, $passed_in);

        $report->tally();
    }

    function testExecuteLegacy_Strings()
    {
        Mock::generatePartial('Plugins_Reports', 'MockReportPlugin_ForExecuteModule_StringTest', array('execute'));

        $module = new ReportExecuteModule();
        $report = new MockReportPlugin_ForExecuteModule_StringTest($this);
        $report->expectOnce('execute', array('One', 'Two', 'Three'));
        $passed_in = array('One', 'Two', 'Three');

        $module->_executeReportWithVariables($report, $passed_in);

        $report->tally();
    }

    function testExecuteLegacy_Dates()
    {
        Mock::generatePartial('Plugins_Reports', 'MockReportPlugin_ForExecuteModule_DateTest', array('execute'));

        $module = new ReportExecuteModule();
        $report = new MockReportPlugin_ForExecuteModule_DateTest($this);
        $monday = new Date('2001-01-01');
        $tuesday = new Date('2001-01-02');
        $wednesday = new Date('2001-01-03');
        $report->expectOnce('execute', array($monday, $tuesday, $wednesday));
        $passed_in = array($monday, $tuesday, $wednesday);

        $module->_executeReportWithVariables($report, $passed_in);

        $report->tally();
    }
}
?>
