<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: OperationInterval.mtp.test.php 4240 2006-02-14 14:45:52Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Maintenance/OperationInterval.php';
require_once MAX_PATH . '/lib/pear/Date.php';

/**
 * A class for testing the
 * Maintenance_Priority_AdServer_RequiredAdvertisementImpression_Normal class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     James Floyd <james@m3.net>
 */
class Maintenance_TestOfOperationInterval extends UnitTestCase
{

    /**
     * The constructor method.
     */
    function Maintenance_TestOfOperationInterval()
    {
        $this->UnitTestCase();
    }

   /**
    * Test 1 - test multiple calls to Singleton
    *   return reference to same object
    */
    function testSingleton()
    {
        // Test  1
        $obj1 = & $this->_getInstance();
        $obj2 = & $this->_getInstance();
        $this->assertIdentical($obj1, $obj2);
    }

   /**
    * Test 1 - test that operation interval from conf is
    *   correctly parsed by default
    */
    function testOperationInterval()
    {
        $operationInterval = & $this->_getInstance();
        $this->assertEqual($operationInterval->interval, $GLOBALS['_MAX']['CONF']['maintenance']['operationInterval']);
    }

   /**
    * Test 1 - test return value is a PEAR::Date object
    * Test 2 - test default returned object is date now
    * Test 3 - test predefined date is correctly created
    */
    function test_getDate()
    {
        // Test 1
        $obj1 = & $this->_getInstance();
        $oDate = $obj1->_getDate();
        $this->assertIsA($oDate, 'Date');

        // Test 2
        $oDate = $obj1->_getDate();
        $oDateNow = new Date();
        $this->assertTrue($oDate->equals($oDateNow));

        // Test 3
        $date = '2005-05-03 11:33:33';
        $oDate = $obj1->_getDate($date);
        $this->assertEqual($oDate->getYear(), 2005);
        $this->assertEqual($oDate->getMonth(), 5);
        $this->assertEqual($oDate->getDay(), 3);
        $this->assertEqual($oDate->getHour(), 11);
        $this->assertEqual($oDate->getMinute(), 33);
        $this->assertEqual($oDate->getSecond(), 33);
    }

   /**
    * The following tests manipulate the private interval property
    * $this->interval of the class to test different interval values.
    * This is easier than manipulating INI settings
    *
    * For invalid intervals, check PEAR::Error object is returned
    *
    * Test 1 - test validation of zero interval
    * Test 2 - test valid value of 1
    * Test 3 - test valid value of 2
    * Test 4 - test valid value of 3
    * Test 5 - test invalid value of 9 returns pear error object
    * Test 6 - test valid vale of 60
    * Test 7 - test invalid value 61 returns pear error object
    * Test 8 - test invalid value of (MINUTES_PER_DAY + 1) returns pear error object
    */
    function testIsValid()
    {
        PEAR::pushErrorHandling(null);
        $operationInterval = & $this->_getInstance();

        // Test 1
        // Manipulate value of interval
        $operationInterval->interval = 0;
        $this->assertIsA($operationInterval->isValid(), 'pear_error');

        // Test 2
        $operationInterval->interval = 1;
        $this->assertIdentical($operationInterval->isValid(), true);

        // Test 3
        $operationInterval->interval = 2;
        $this->assertIdentical($operationInterval->isValid(), true);

        // Test 4
        $operationInterval->interval = 3;
        $this->assertIdentical($operationInterval->isValid(), true);

        // Test 5
        $operationInterval->interval = 9;
        $this->assertIsA($operationInterval->isValid(), 'pear_error');

        // Test 6
        $operationInterval->interval = 60;
        $this->assertIdentical($operationInterval->isValid(), true);

        // Test 7
        $operationInterval->interval = 61;
        $this->assertIsA($operationInterval->isValid(), 'pear_error');

        // Test 8
        $operationInterval->interval = MINUTES_PER_DAY + 1;
        $this->assertIsA($operationInterval->isValid(), 'pear_error');

        // kill object
        unset($operationInterval);
        PEAR::popErrorHandling();
    }

    /**
    * Test 1 - set up valid start date and add one interval
    * Test 2 - set up valid start date and add 24 intervals
    * Test 3 - set up start and end with same date time returns zero
    * Test 4 - set up start and end with end before start check returns zero
    * Test 5 - invalid argument pass - no end date object
    * Test 6 - invalid argument pass - no start date
    * Test 7 - invalid argument pass - no start or end date
    */
    function testGetRemaining()
    {
        PEAR::pushErrorHandling(null);
        $operationInterval = & $this->_getInstance();

        // Test 1
        $oStartDate = new Date('2005-05-03 09:00:00');
        $oEndDate = new Date();
        $oEndDate->copy($oStartDate);
        $oEndDate->addSeconds(($operationInterval->interval * 60));
        $this->assertEqual(1, $operationInterval->getRemaining($oStartDate, $oEndDate));

        // Test 2
        $oStartDate = new Date('2005-05-03 09:00:00');
        $oEndDate = new Date();
        $oEndDate->copy($oStartDate);
        $oEndDate->addSeconds(24 * ($operationInterval->interval * 60));
        $this->assertEqual(24, $operationInterval->getRemaining($oStartDate, $oEndDate));

        // Test 3
        $oStartDate = new Date('2005-05-03 09:00:00');
        $oEndDate = new Date();
        $oEndDate->copy($oStartDate);
        $this->assertEqual(0, $operationInterval->getRemaining($oStartDate, $oEndDate));

        // Test 4
        $oStartDate = new Date('2005-05-03 09:00:00');
        $oEndDate = new Date();
        $oEndDate->copy($oStartDate);
        $oStartDate->addSeconds(24 * ($operationInterval->interval * 60));
        $this->assertEqual(0, $operationInterval->getRemaining($oStartDate, $oEndDate));

        // Test 5
        $oStartDate = new Date('2005-05-03 09:00:00');
        $endDate = '';
        $this->assertIdentical(true, $this->assertIsA($operationInterval->getRemaining($oStartDate, $endDate), 'Pear_Error'));

        // Test 6
        $oEndDate = new Date('2005-05-03 09:00:00');
        $startDate = '';
        $this->assertIdentical(true, $this->assertIsA($operationInterval->getRemaining($startDate, $oEndDate), 'Pear_Error'));

        // Test 7
        $startDate = '';
        $endDate = '';
        $this->assertIdentical(true, $this->assertIsA($operationInterval->getRemaining($startDate, $oEndDate), 'Pear_Error'));
        PEAR::popErrorHandling();
    }

    function testConvertDateToOperationIntervalStartAndEndDates()
    {
        $obj = &$this->_getInstance();
        $obj->interval = 60;
        $date = new Date('2004-08-08 00:40:00');
        $aLimits = $obj->convertDateToOperationIntervalStartAndEndDates($date);
        $this->assertEqual($aLimits['start'], new Date('2004-08-08 00:00:00'));
        $this->assertEqual($aLimits['end'], new Date('2004-08-08 00:59:59'));
        unset($obj);

        $obj = &$this->_getInstance();
        $obj->interval = 30;
        $date = new Date('2004-08-08 00:40:00');
        $aLimits = $obj->convertDateToOperationIntervalStartAndEndDates($date);
        $this->assertEqual($aLimits['start'], new Date('2004-08-08 00:30:00'));
        $this->assertEqual($aLimits['end'], new Date('2004-08-08 00:59:59'));
        unset($obj);
    }

    function testConvertDateToPreviousOperationIntervalStartAndEndDates()
    {
        $obj = &$this->_getInstance();
        $obj->interval = 60;
        $date = new Date('2004-08-08 00:40:00');
        $aLimits = $obj->convertDateToPreviousOperationIntervalStartAndEndDates($date);
        $this->assertEqual($aLimits['start'], new Date('2004-08-07 23:00:00'));
        $this->assertEqual($aLimits['end'], new Date('2004-08-07 23:59:59'));
        unset($obj);

        $obj = &$this->_getInstance();
        $obj->interval = 30;
        $date = new Date('2004-08-08 00:40:00');
        $aLimits = $obj->convertDateToPreviousOperationIntervalStartAndEndDates($date);
        $this->assertEqual($aLimits['start'], new Date('2004-08-08 00:00:00'));
        $this->assertEqual($aLimits['end'], new Date('2004-08-08 00:29:59'));
        unset($obj);
    }

    function testGetIntervalIdRange()
    {
        $obj = &$this->_getInstance();
        $obj->interval = 60;

        $range = array();
        $date = new Date('2004-09-11 19:00:00');
        $range = $obj->getIntervalIdRange($date, 5);
        $this->assertNull($range[162]);
        $this->assertEqual($range[163]['start'], new Date('2004-09-11 19:00:00'));
        $this->assertEqual($range[163]['end'], new Date('2004-09-11 19:59:59'));
        $this->assertEqual($range[164]['start'], new Date('2004-09-11 20:00:00'));
        $this->assertEqual($range[164]['end'], new Date('2004-09-11 20:59:59'));
        $this->assertEqual($range[165]['start'], new Date('2004-09-11 21:00:00'));
        $this->assertEqual($range[165]['end'], new Date('2004-09-11 21:59:59'));
        $this->assertEqual($range[166]['start'], new Date('2004-09-11 22:00:00'));
        $this->assertEqual($range[166]['end'], new Date('2004-09-11 22:59:59'));
        $this->assertEqual($range[167]['start'], new Date('2004-09-11 23:00:00'));
        $this->assertEqual($range[167]['end'], new Date('2004-09-11 23:59:59'));
        $this->assertNull($range[0]);
        unset($obj);
    }

   /**
    * Test 1 - 10 day range
    * Test 2 - 1 day range
    * Test 3 - 1 day range by specifying last hour
    */
    function testGetDayRange()
    {
        $obj = &$this->_getInstance();
        // Test 1
        $oStart = new Date('2005-05-01 00:00:00');
        $oEnd = new Date('2005-05-10 23:59:59');
        $aRange = $obj->getDayRange($oStart, $oEnd);

        $this->assertEqual(10, count($aRange));
        $this->assertEqual($aRange[0][0], new Date('2005-05-01 00:00:00'));
        $this->assertEqual($aRange[1][0], new Date('2005-05-02 00:00:00'));
        $this->assertEqual($aRange[2][0], new Date('2005-05-03 00:00:00'));
        $this->assertEqual($aRange[3][0], new Date('2005-05-04 00:00:00'));
        $this->assertEqual($aRange[4][0], new Date('2005-05-05 00:00:00'));
        $this->assertEqual($aRange[5][0], new Date('2005-05-06 00:00:00'));
        $this->assertEqual($aRange[6][0], new Date('2005-05-07 00:00:00'));
        $this->assertEqual($aRange[7][0], new Date('2005-05-08 00:00:00'));
        $this->assertEqual($aRange[8][0], new Date('2005-05-09 00:00:00'));
        $this->assertEqual($aRange[9][0], new Date('2005-05-10 00:00:00'));
        unset($obj);


        // Test 2
        $obj = &$this->_getInstance();
        $oStart = new Date('2005-05-01 00:00:00');
        $oEnd = new Date('2005-05-01 23:59:59');
        $aRange = $obj->getDayRange($oStart, $oEnd);
        $this->assertEqual(1, count($aRange));
        $this->assertEqual($aRange[0][0], new Date('2005-05-01 00:00:00'));
        unset($obj);


        // Test 3
        $obj = &$this->_getInstance();
        $oStart = new Date('2005-05-10 22:59:59');
        $oEnd = new Date('2005-05-10 23:59:59');
        $aRange = $obj->getDayRange($oStart, $oEnd);

        $this->assertEqual(1, count($aRange));
        $this->assertEqual($aRange[0][0], new Date('2005-05-10 00:00:00'));
        unset($obj);
    }

   /**
    * Test 1 - Test start of week interval = 0
    * Test 2 - Test start of week + 1 OI returns intervalID = 1
    * Test 3 - interval of 30 mins
    */
    function testGetIntervalID()
    {
        // Test 1
        $obj = &$this->_getInstance();
        // force interval to be 60 minutes
        $obj->interval = 60;
        // Test a date in the first operation interval ID in the week before the test was
        // written, using a default operation interval of 60 minutes
        $date = new Date('2004-08-08 00:40:00');
        $result = $obj->getIntervalID($date);
        $this->assertEqual($result, 0);

        // Test 2
        $date->addSeconds(($obj->interval * 60));
        $result = $obj->getIntervalID($date);
        $this->assertEqual($result, 1);

        // Test 3
        $obj->interval = 30;
        // Test a date in the last operation interval ID in the week before the test was
        // written, using an operation interval of 30 minutes
        $date = new Date('2004-08-14 23:40:00');
        $result = $obj->getIntervalID($date);
        $this->assertEqual($result, 335);
        unset($obj);
    }

    function testGetNextIntervalID()
    {
        $obj = & $this->_getInstance();
        $lastIntervalID = ((MINUTES_PER_WEEK / $obj->interval) -1);
        // Test 1
        $this->assertEqual(0, $obj->getNextIntervalID($lastIntervalID));
        // Test 2
        $this->assertEqual(3, $obj->getNextIntervalID(2));
    }

   /**
    * Test 1 - check intervalID zero returns last OI of week
    * Test 2 - check intervalID 2 returns 1
    */
    function testGetPreviousIntervalID()
    {
        $obj = & $this->_getInstance();
        $lastIntervalID = ((MINUTES_PER_WEEK / $obj->interval) -1);
        // Test 1
        $this->assertEqual($lastIntervalID, $obj->getPreviousIntervalID(0));

        // Test 2
        $this->assertEqual(1, $obj->getPreviousIntervalID(2));
    }

   /**
    * Test 1 -  Test adding 5 intervals to last in week
    * Test 2 -  Test adding 2 weeks worth of intervals to last
    *           interval in week
    */
    function testIncrementIntervalId()
    {
        $obj = & $this->_getInstance();

        // Test 1
        $obj->interval = 60;
        $intervalId = 167;
        $intervals = 5;

        $incrementedId = $obj->incrementIntervalID($intervalId, $intervals);
        $this->assertTrue(($incrementedId == 4));
        $this->assertFalse(is_a($incrementedId, 'PEAR'));

        // Test 2
        $obj->interval = 60;
        $intervalId = 167;
        $intervals = 336;

        $incrementedId = $obj->incrementIntervalID($intervalId, $intervals);
        $this->assertTrue(($incrementedId == 167));
        $this->assertFalse(is_a($incrementedId, 'PEAR'));
    }

    /**
    * Test 1 -  Test subtracting 5 intervals to last in week
    * Test 2 -  Test subtracting 2 weeks worth of intervals to last
    *           interval in week
    */
    function testDecrementIntervalId()
    {
        $obj = & $this->_getInstance();

        // Test 1
        $obj->interval = 60;
        $intervalId = 167;
        $intervals = 5;

        $incrementedId = $obj->decrementIntervalID($intervalId, $intervals);
        $this->assertTrue(($incrementedId == 162));
        $this->assertFalse(is_a($incrementedId, 'PEAR'));

        // Test 2
        $obj->interval = 60;
        $intervalId = 167;
        $intervals = 336;

        $incrementedId = $obj->decrementIntervalID($intervalId, $intervals);
        $this->assertTrue(($incrementedId == 167));
        $this->assertFalse(is_a($incrementedId, 'PEAR'));
    }

    function testGetNext()
    {
        // testing with mock is a bit pointless as only seconds are added
        $date = '2005-05-03 00:00:00';
        $obj = &$this->_getInstance();
        $obj->interval = 60;
        $aNext = $obj->getNext($date);

        $expectedStart = new Date('2005-05-03 01:00:00');
        $expectedEnd = new Date('2005-05-03 01:59:59');

        $this->assertEqual($expectedStart, $aNext['start']);
        $this->assertEqual($expectedEnd, $aNext['end']);
        unset($obj);
    }

    function testIntervalsPerDay()
    {
        $obj = & $this->_getInstance();
        $this->assertEqual((SECONDS_PER_DAY / ($obj->interval * SECONDS_PER_MINUTE)), $obj->intervalsPerDay());
        unset($obj);
    }

    function testIntervalsPerWeek()
    {
        $obj = & $this->_getInstance();
        $obj->interval = 60;
        $this->assertEqual((7 * 24), $obj->intervalsPerWeek());
        unset($obj);
    }

    function testSecondsPerInterval()
    {
        $obj = & $this->_getInstance();
        $this->assertEqual(($obj->interval * SECONDS_PER_MINUTE), $obj->secondsPerInterval());
        unset($obj);
    }

    function testGetRemainingByDate()
    {
        // Test 1
        $obj = & $this->_getInstance();
        $obj->interval = 60;
        $oDate = new Date('2005-05-03 00:00:00');
        $this->assertEqual(24, $obj->getRemainingByDate($oDate));
        unset($obj);

        // Test 2
        $obj = & $this->_getInstance();
        $obj->interval = 30;
        $oDate = new Date('2005-05-03 00:00:00');
        $this->assertEqual(48, $obj->getRemainingByDate($oDate));
        unset($obj);

        // Test 3
        $obj = & $this->_getInstance();
        $obj->interval = 60;
        $oDate = new Date('2005-05-03 23:59:59');
        $this->assertEqual(0, $obj->getRemainingByDate($oDate));
        unset($obj);

        // Test 3
        $obj = & $this->_getInstance();
        $obj->interval = 60;
        $oDate = new Date('2005-05-03 23:00:00');
        $this->assertEqual(1, $obj->getRemainingByDate($oDate));
        unset($obj);
    }

    function testGetDayIntervalNumber()
    {
        $obj = & $this->_getInstance();
        $obj->interval = 60;

        $oDate = new Date('2005-05-03 00:00:00');
        $this->assertEqual(0, $obj->getDayIntervalNumber($oDate));

        $oDate = new Date('2005-05-03 05:00:00');
        $this->assertEqual(5, $obj->getDayIntervalNumber($oDate));

        $oDate = new Date('2005-05-03 23:00:00');
        $this->assertEqual(23, $obj->getDayIntervalNumber($oDate));
        unset($obj);
    }

    function &_getInstance()
    {
        return OperationInterval::singleton();
    }

}

?>
