<?php
/**
 * Date range
 *
 * @since 0.3.19 - Feb 13, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id:DateRange.core.test.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 */

require_once('Date.php');
require_once(MAX_PATH . '/lib/max/DateRange.php');

class DateRangeTest extends UnitTestCase
{
    function testTodayFactory()
    {
        $now = new Date('1987-04-12 15:10:11');
        $today_start = new Date('1987-04-12 00:00:00');
        $today_end = new Date('1987-04-13 00:00:00');

        $range = DateRange::newToday($now);

        $this->assertEqual($range->getStartDate(), $today_start);
        $this->assertEqual($range->getEndDate(), $today_end, '"Today" ends at midnight tonight, which is the instant tomorrow begins. %s');
    }

    function testUseLastWeek()
    {
        $now = new Date('2000-01-01 00:20:40');
        $week_start = new Date('1999-12-19 00:00:00');
        $week_end = new Date('1999-12-26 00:00:00');

        $range = new DateRange($now);
        $range->useLastWeek();

        $this->assertEqual($range->getStartDate(), $week_start);
        $this->assertEqual($range->getEndDate(), $week_end, '"Last week" ends on midnight spanning Saturday/Sunday. %s');
    }

    function testUseLast7Days()
    {
        $now = new Date('2000-01-19 15:20:40');
        $week_start = new Date('2000-01-12 00:00:00');
        $week_end = new Date('2000-01-19 00:00:00');

        $range = new DateRange($now);
        $range->useLast7Days();

        $this->assertEqual($range->getStartDate(), $week_start);
        $this->assertEqual($range->getEndDate(), $week_end);
    }

    function testUseLastMonth()
    {
        $now = new Date('2000-01-19 15:20:40');
        $month_start = new Date('1999-12-01 00:00:00');
        $month_end = new Date('2000-01-01 00:00:00');

        $range = new DateRange($now);
        $range->useLastMonth();

        $this->assertEqual($range->getStartDate(), $month_start);
        $this->assertEqual($range->getEndDate(), $month_end);
    }

    function testGetStartSql()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $sql = $range->getStartSql();
        $this->assertEqual($sql, "'2000-01-19 00:00:00'");
    }

    function testGetEndSql_Seconds()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $sql = $range->getEndSqlForComparingTimestamps();
        $this->assertEqual($sql, "'2000-01-20 00:00:00'");
    }

    function testGetEndSql_Days()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $sql = $range->getEndSqlForComparingDays();
        $this->assertEqual($sql, "'2000-01-19'");
    }

    function testGetStartDateForDisplay()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $display = $range->getStartDateForDisplay();
        $this->assertEqual($display, '19/01/2000');
    }

    function testGetEndDateForDisplay()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $display = $range->getEndDateForDisplay();
        $this->assertEqual($display, '19/01/2000');
    }
}
?>
